@@ -32,6 +32,8 @@ char acct_sccsid[] =
 #include	"radutmp.h"
 #endif
 
+extern char* accounting_method;
+
 /*
  *	FIXME: this should be configurable.
  */
@@ -70,7 +72,7 @@ char *uue(void *in) {
 }
 
 
-int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f, SQLSOCK *socket) {
+int rad_accounting_sql(AUTH_REQ *authreq, int authtype, char *f, SQLSOCK *socket) {
 	time_t		nowtime;
 	struct tm	*tim;
         char		datebuf[20];
@@ -243,6 +245,79 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f, SQLSOCK *socke
 	return ret;
 }
 
+int rad_accounting_file(AUTH_REQ *authreq, int authtype, char *f) 
+{
+        FILE            *outfd;
+        char            nasname[128];
+        char            buffer[512];
+        char            *s;
+        VALUE_PAIR      *pair;
+        UINT4           nas;
+        NAS             *cl;
+	struct stat     st;
+
+        /*
+         *      See if we have an accounting directory. If not,
+         *      return.
+         */
+        if (stat(radacct_dir, &st) < 0)
+                return 0;
+
+        /*
+         *      Find out the name of this terminal server. We try
+         *      to find the PW_NAS_IP_ADDRESS in the naslist file.
+         *      If that fails, we look for the originating address.
+         *      Only if that fails we resort to a name lookup.
+         */
+        cl = NULL;
+        nas = authreq->ipaddr;
+        if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
+                nas = pair->lvalue;
+        if (authreq->server_ipaddr)
+                nas = authreq->server_ipaddr;
+
+       if ((cl = nas_find(nas)) != NULL) {
+                /* shortname and longname are < 128 bytes */
+                if (cl->shortname[0])
+                        strcpy(nasname, cl->shortname);
+                else
+                        strcpy(nasname, cl->longname);
+        }
+
+        if (cl == NULL) {
+                s = ip_hostname(nas);
+                if (strlen(s) >= sizeof(nasname) || strchr(s, '/'))
+                        return -1;
+                strcpy(nasname, s);
+        }
+ 
+	/*
+         *      Create a directory for this nas.
+         */
+        sprintf(buffer, "%.250s/%.250s", radacct_dir, nasname);
+        (void) mkdir(buffer, 0755);
+
+        /*
+         *      Write Detail file.
+         */
+        sprintf(buffer, "%.200s/%.200s/%.100s", radacct_dir, nasname, f ? f : "detail");
+        if((outfd = fopen(buffer, "a")) == (FILE *)NULL) {
+	   log(L_ERR, "Acct: Couldn't open file %s", buffer);
+	   return -1;
+        } else {
+	   /* print timestamp */
+	   fprintf(outfd, "%ld,", time(NULL));
+
+	   /* Write each attribute/value to the log file */
+	   pair = authreq->request;
+	   fprint_attr_list_csv(outfd, pair, 0);
+	   fputs("\n", outfd);
+	   fclose(outfd);
+	}
+
+	return 0;
+}
+  
 
 /*
  *	rad_accounting: call both the old and new style accounting functions.
@@ -264,8 +339,15 @@ int rad_accounting(AUTH_REQ *authreq, int activefd, SQLSOCK *socket)
 		return -1;
 	}
 
-	if (rad_accounting_orig(authreq, auth, NULL, socket) == 0)
-		reply = 1;
+	if (strchr(accounting_method, 'f') != NULL) {
+	   if (rad_accounting_file(authreq, auth, NULL) == 0)
+	      reply = 1;
+	}
+
+	if (strchr(accounting_method, 's') != NULL) {	   
+	   if (rad_accounting_sql(authreq, auth, NULL, socket))
+	      reply = 1;
+	}
 
 	if (reply) {
 		/*
