@@ -529,7 +529,6 @@ int rad_auth_init(AUTH_REQ *authreq, int activefd) {
 	 */
 	hints_setup(authreq);
 
-/* FIXME: find out what this logs */
 /*	if (log_auth_detail)
 		rad_accounting_orig(authreq, -1, "detail.auth");
 */
@@ -576,10 +575,12 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 
 	/*
 	 *	Get the username from the request.
-	 *	All checking has been done by rad_auth_init().
+	 *	All length checking has been done by rad_auth_init().
 	 */
 	namepair = pairfind(authreq->request, PW_USER_NAME);
 
+	DEBUG2("Username is now %s", namepair->strvalue);
+
 	/*
 	 *	FIXME: we calculate the digest twice ...
 	 *	once here and once in rad_auth_init()
@@ -796,6 +797,100 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 
 			}
 
+			if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_DAILY_TIME_LIMIT)) != NULL) {
+
+				SQL_ROW		row;
+				char		querystr[256];
+				unsigned long	allowed;
+				unsigned long	diff;
+				unsigned long	used = 0;
+	
+				allowed = check_item->lvalue;
+				sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE Realm = '%s' and TO_DAYS(AcctStartTime) = TO_DAYS(NOW())", authreq->realm);
+				sql_select_query(socket, querystr);
+				row = sql_fetch_row(socket);
+				if (row[0] == NULL)
+					used = 0;
+				else
+					used = atol(row[0]);
+				sql_finish_select_query(socket);
+				
+				if (used >= allowed) {
+					sprintf(umsg, "\r\nYour realm has reached it's limit of %d seconds\r\n\n", (int)check_item->lvalue);
+					user_msg = umsg;
+					result = -1;
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
+					log(L_ERR, "Realm usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
+				}
+			
+				if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
+					diff = allowed - used;
+					if (diff < reply_item->lvalue)
+						reply_item->lvalue = diff;
+				} else if (result >= 0) {
+					if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+						log(L_ERR|L_CONS, "no memory");
+						exit(1);
+					}
+					memset(reply_item, 0, sizeof(VALUE_PAIR));
+					reply_item->attribute = PW_SESSION_TIMEOUT;
+					strcpy(reply_item->name, "Session-Timeout");
+					reply_item->type = PW_TYPE_INTEGER;
+					reply_item->length = 4;
+					reply_item->lvalue = allowed - used;
+					pairadd(&authreq->reply_pairs, reply_item);
+				}
+
+			}
+
+			if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_WEEKLY_TIME_LIMIT)) != NULL) {
+
+				SQL_ROW		row;
+				char		querystr[256];
+				unsigned long	allowed;
+				unsigned long	diff;
+				unsigned long	used = 0;
+	
+				allowed = check_item->lvalue;
+				sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE Realm = '%s' and YEAR(AcctStartTime) = YEAR(NOW()) AND WEEK(AcctStartTime,1) = WEEK(NOW(), 1)", authreq->realm);
+				sql_select_query(socket, querystr);
+				row = sql_fetch_row(socket);
+				if (row[0] == NULL)
+					used = 0;
+				else
+					used = atol(row[0]);
+				sql_finish_select_query(socket);
+				
+				if (used >= allowed) {
+					sprintf(umsg, "\r\nYour realm has reached it's limit of %d seconds\r\n\n", (int)check_item->lvalue);
+					user_msg = umsg;
+					result = -1;
+					rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
+					log(L_ERR, "Realm usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, authreq->realm, auth_name(authreq, 1));
+				}
+			
+				if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
+					diff = allowed - used;
+					if (diff < reply_item->lvalue)
+						reply_item->lvalue = diff;
+				} else if (result >= 0) {
+					if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+						log(L_ERR|L_CONS, "no memory");
+						exit(1);
+					}
+					memset(reply_item, 0, sizeof(VALUE_PAIR));
+					reply_item->attribute = PW_SESSION_TIMEOUT;
+					strcpy(reply_item->name, "Session-Timeout");
+					reply_item->type = PW_TYPE_INTEGER;
+					reply_item->length = 4;
+					reply_item->lvalue = allowed - used;
+					pairadd(&authreq->reply_pairs, reply_item);
+				}
+
+			}
+
+
+
 			if (result >= 0 &&
 			   (check_item = pairfind(authreq->check_pairs, PW_LOGIN_TIME)) != NULL) {
 	
@@ -907,6 +1002,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 				return -1;
 			}
 			if (result == -2) {
+				/* FIXME: Handle Auth-Type Reject with a better error message */
 				if ((reply_item = pairfind(authreq->reply_pairs, PW_REPLY_MESSAGE)) != NULL)
 					user_msg = reply_item->strvalue;
 			}
@@ -1082,8 +1178,7 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 				user_msg = "\r\nYou are already logged in - access denied\r\n\n";
 			}
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
-		log(L_ERR, "Multiple logins: [%s] (%s) max. %d%s", namepair->strvalue, auth_name(authreq, 1),
-				check_item->lvalue, r == 2 ? " [MPP attempt]" : "");
+			log(L_ERR, "Multiple logins: [%s] (%s) max. %d%s", namepair->strvalue, auth_name(authreq, 1), check_item->lvalue, r == 2 ? " [MPP attempt]" : "");
 			result = -1;
 #if 0 /* DEBUG */
 			ipno[0] = 0;
@@ -1094,6 +1189,103 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd, SQLSOCK *socket) {
 		}
 	}
 
+	if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_DAILY_TIME_LIMIT)) != NULL) {
+
+		SQL_ROW		row;
+		char		querystr[256];
+		unsigned long	allowed = 0;
+		unsigned long	used = 0;
+		unsigned long	diff = 0;
+
+		allowed = check_item->lvalue;
+		sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' and TO_DAYS(AcctStartTime) = TO_DAYS(NOW())", namepair->strvalue);
+
+		sql_select_query(socket, querystr);
+		row = sql_fetch_row(socket);
+		if (row[0] == NULL)
+			used = 0;
+		else
+			used = atol(row[0]);
+		sql_finish_select_query(socket);
+
+
+		fprintf(stderr, "usertime %d for user %s, allowed %d\n", (int)used, namepair->strvalue, (int)allowed);
+
+		if (used >= allowed) {
+			sprintf(umsg, "\r\nYour account has reached it limit of %d seconds\r\n\n", (int)check_item->lvalue);
+			user_msg = umsg;
+			result = -1;
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
+			log(L_ERR, "Usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, namepair->strvalue, auth_name(authreq, 1));
+		}
+			
+		if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
+			diff = allowed - used;
+			if (diff < reply_item->lvalue)
+				reply_item->lvalue = diff;
+		} else {
+			if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+				log(L_ERR|L_CONS, "no memory");
+				exit(1);
+			}
+			memset(reply_item, 0, sizeof(VALUE_PAIR));
+			reply_item->attribute = PW_SESSION_TIMEOUT;
+			strcpy(reply_item->name, "Session-Timeout");
+			reply_item->type = PW_TYPE_INTEGER;
+			reply_item->length = 4;
+			reply_item->lvalue = allowed - used;
+			pairadd(&authreq->reply_pairs, reply_item);
+		}
+
+	}
+
+	if (result >=0 && (check_item = pairfind(authreq->check_pairs, PW_WEEKLY_TIME_LIMIT)) != NULL) {
+
+		SQL_ROW		row;
+		char		querystr[256];
+		unsigned long	allowed = 0;
+		unsigned long	used = 0;
+		unsigned long	diff = 0;
+
+		allowed = check_item->lvalue;
+		sprintf(querystr, "SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' and YEAR(AcctStartTime) = YEAR(NOW()) AND WEEK(AcctStartTime,1) = WEEK(NOW(), 1)", namepair->strvalue);
+
+		sql_select_query(socket, querystr);
+		row = sql_fetch_row(socket);
+		if (row[0] == NULL)
+			used = 0;
+		else
+			used = atol(row[0]);
+		sql_finish_select_query(socket);
+
+		if (used >= allowed) {
+			sprintf(umsg, "\r\nYour account has reached it limit of %d seconds\r\n\n", (int)check_item->lvalue);
+			user_msg = umsg;
+			result = -1;
+			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq, authreq->reply_pairs, user_msg, activefd);
+			log(L_ERR, "Usage limit of %d seconds reached: [%s] (%s)", (int)check_item->lvalue, namepair->strvalue, auth_name(authreq, 1));
+		}
+			
+		if (result >=0 && (reply_item = pairfind(authreq->reply_pairs, PW_SESSION_TIMEOUT)) != NULL) {
+			diff = allowed - used;
+			if (diff < reply_item->lvalue)
+				reply_item->lvalue = diff;
+		} else {
+			if (!(reply_item = malloc(sizeof(VALUE_PAIR)))){
+				log(L_ERR|L_CONS, "no memory");
+				exit(1);
+			}
+			memset(reply_item, 0, sizeof(VALUE_PAIR));
+			reply_item->attribute = PW_SESSION_TIMEOUT;
+			strcpy(reply_item->name, "Session-Timeout");
+			reply_item->type = PW_TYPE_INTEGER;
+			reply_item->length = 4;
+			reply_item->lvalue = allowed - used;
+			pairadd(&authreq->reply_pairs, reply_item);
+		}
+
+	}
+
 	if (result >= 0 &&
 	   (check_item = pairfind(authreq->check_pairs, PW_LOGIN_TIME)) != NULL) {
 
