@@ -425,6 +425,8 @@ int paircmp(VALUE_PAIR *request, VALUE_PAIR *check) {
 			case PW_ACTIVATION:
 			case PW_RADIUS_OPERATOR:
 			case PW_STRIP_USERNAME:
+			case PW_DAILY_TIME_LIMIT:
+			case PW_WEEKLY_TIME_LIMIT:
 				check_item = check_item->next;
 				continue;
 		}
@@ -1185,6 +1187,18 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 
 			switch(pair->type) {
 
+#if defined( BINARY_FILTERS )
+			case PW_TYPE_FILTER_BINARY:
+				/*
+				* special case to convert filter to binary
+				*/
+				if ( filterBinary( pair, valstr ) == -1 ) {
+					free(pair);
+					return(-1);
+				}
+				break;
+#endif	/* BINARY_FILTERS */
+
 			case PW_TYPE_STRING:
 				/* Pair->strvalue has been set already */
 				pair->length = strlen(pair->strvalue);
@@ -1643,7 +1657,11 @@ REALM *realm_find(char *realm)
 	REALM *cl;
 
 	for(cl = realms; cl; cl = cl->next)
+#if defined(NOCASEREALMS)
+		if (strcasecmp(cl->realm, realm) == 0)
+#else
 		if (strcmp(cl->realm, realm) == 0)
+#endif /* NOCASEREALMS */
 			break;
 	if (cl) return cl;
 	for(cl = realms; cl; cl = cl->next)
