@@ -296,9 +296,10 @@ int sql_init(int reload) {
        
        log(L_INFO,"SQL: Attempting to connect to %s@%s:%s", sql->config->sql_login, sql->config->sql_server, sql->config->sql_db);
 
-       sql_init_socket(reload);
-           
-       return 0;
+	if (sql_init_socket(reload))
+		return 1;
+	else
+		return 0;
 }
 
 
@@ -314,12 +315,13 @@ int sql_init_socket(int reload) {
 	int i;
 
 	/* Clear up old connections if reload */
-	if (reload)
+	if (reload) {
 	        for (i = 0; i < sql->config->max_sql_oldsocks; i++) {
 			if (!sql_close_socket(sql->socks[i]))
 				log(L_CONS|L_ERR, "Could not release socket %d", i);
 		}
 		free(sql->socks);
+	}
 	
 
         /* Initalize our connection pool */
@@ -330,10 +332,6 @@ int sql_init_socket(int reload) {
 	}
 	bzero(sql->socks,sizeof(SQLSOCK *) * sql->config->max_sql_socks);
         for (i = 0; i < sql->config->max_sql_socks; i++) {
-		sql->socks[i] = malloc(sizeof(SQLSOCK));
-		if(!sql->socks[i])
-			log(L_ERR,"Unable to allocate memory for sockets structure");
-		bzero(sql->socks[i],sizeof(SQLSOCK));
 	 	if ((sql->socks[i] = sql_create_socket()) == NULL) {
 	   		log(L_CONS|L_ERR, "SQL: Failed to connect socket %d", i);
 		} else {
@@ -358,6 +356,7 @@ int sql_close_socket(SQLSOCK *socket) {
 	DEBUG2("SQL: Closing socket %d", socket->id);
 	sql_close(socket);
 	free(socket);
+
 	return 1;
 }
  
@@ -430,22 +429,22 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 	char		fixeduser[AUTH_STRING_LEN*2+1];
 	FILE		*mysqlfile;
 	int		num = 0;
+	int		ret = 1;
 #ifdef NT_DOMAIN_HACK
 	char		*ptr;
 	char		newname[AUTH_STRING_LEN];
 #endif
 	
 
-
-     if((mysqlfile = fopen(QUERYLOG, "a")) == (FILE *)NULL) {
-            log(L_ERR, "Acct: Couldn't open file %s", QUERYLOG);
-     } else { 
-        #if defined(F_LOCK) && !defined(BSD)
-              (void)lockf((int)mysqlfile, (int)F_LOCK, (off_t)SQL_LOCK_LEN);
-        #else
-              (void)flock(mysqlfile, SQL_LOCK_EX);
-        #endif
-     }
+	if ((mysqlfile = fopen(QUERYLOG, "a")) == (FILE *)NULL) {
+		log(L_ERR, "Acct: Couldn't open file %s", QUERYLOG);
+	} else { 
+#if defined(F_LOCK) && !defined(BSD)
+		(void)lockf((int)mysqlfile, (int)F_LOCK, (off_t)SQL_LOCK_LEN);
+#else
+		(void)flock(mysqlfile, SQL_LOCK_EX);
+#endif
+	}
 
 #ifdef NT_DOMAIN_HACK
 	/*
@@ -461,34 +460,22 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 	}
 #endif /* NT_DOMAIN_HACK */
 
-	// keep garbled usernames from messing up our database queries
+	/* keep garbled usernames from messing up our database queries */
 	sql_escape_string(socket, fixeduser, sqlrecord->UserName, strlen(sqlrecord->UserName));
 	strNcpy(sqlrecord->UserName, fixeduser, SQLBIGREC);
 
-     if (sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
-        log(L_INFO, "Portmaster %s rebooted at %s", sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
+	if (sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_ON || sqlrecord->AcctStatusTypeId == PW_STATUS_ACCOUNTING_OFF) {
+		log(L_INFO, "NAS %s rebooted at %s", sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
   
-         /* The Terminal server informed us that it was rebooted
-         * STOP all records from this NAS */
+		/* The Terminal server informed us that it was rebooted
+		 * STOP all records from this NAS */
 
-         sprintf(querystr, "UPDATE %s SET AcctStopTime='%s', AcctSessionTime=unix_timestamp('%s') - unix_timestamp(AcctStartTime), AcctTerminateCause='%s', AcctStopDelay = %ld WHERE AcctSessionTime=0 AND AcctStopTime=0 AND NASIPAddress= '%s' AND AcctStartTime <= '%s'", sql->config->sql_acct_table, sqlrecord->AcctTimeStamp, sqlrecord->AcctTimeStamp, sqlrecord->AcctTerminateCause, sqlrecord->AcctDelayTime, sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
+		sprintf(querystr, "UPDATE %s SET AcctStopTime='%s', AcctSessionTime=unix_timestamp('%s') - unix_timestamp(AcctStartTime), AcctTerminateCause='%s', AcctStopDelay = %ld WHERE AcctSessionTime=0 AND AcctStopTime=0 AND NASIPAddress= '%s' AND AcctStartTime <= '%s'", sql->config->sql_acct_table, sqlrecord->AcctTimeStamp, sqlrecord->AcctTimeStamp, sqlrecord->AcctTerminateCause, sqlrecord->AcctDelayTime, sqlrecord->NASIPAddress, sqlrecord->AcctTimeStamp);
 
-       	 if (sql_query(socket, querystr) < 0)
-	      log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", sql_error(socket));
-	 sql_finish_query(socket);
-
-         if (mysqlfile) {
-              fputs(querystr, mysqlfile);
-              fputs(";\n", mysqlfile);
-              fclose(mysqlfile);
-          }
-          return 0;
-      } 
-
-	if (sqlrecord->AcctStatusTypeId == PW_STATUS_ALIVE) {
-		sprintf(querystr, "UPDATE %s SET FramedIPAddress = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress= '%s'", sql->config->sql_acct_table, sqlrecord->FramedIPAddress, sqlrecord->AcctSessionId, sqlrecord->UserName, sqlrecord->NASIPAddress);
-		if (sql_query(socket, querystr) < 0)
-			log(L_ERR, "Acct: Couldn't update SQL accounting for ALIVE packet - %s", sql_error(socket));
+		if (!sql_query(socket, querystr)) {
+			log(L_ERR, "Acct: Couldn't update SQL accounting after NAS reboot - %s", sql_error(socket));
+			ret = 0;
+		}
 		sql_finish_query(socket);
 
 		if (mysqlfile) {
@@ -496,139 +483,172 @@ int sql_save_acct(SQLSOCK *socket, SQLREC *sqlrecord) {
 			fputs(";\n", mysqlfile);
 			fclose(mysqlfile);
 		}
-		return 0;
+		return ret;
+	} 
+
+	if (sqlrecord->AcctStatusTypeId == PW_STATUS_ALIVE) {
+		sprintf(querystr, "UPDATE %s SET FramedIPAddress = '%s' WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress= '%s'", sql->config->sql_acct_table, sqlrecord->FramedIPAddress, sqlrecord->AcctSessionId, sqlrecord->UserName, sqlrecord->NASIPAddress);
+		if (!sql_query(socket, querystr)) {
+			log(L_ERR, "Acct: Couldn't update SQL accounting for ALIVE packet - %s", sql_error(socket));
+			ret = 0;
+		}
+		sql_finish_query(socket);
+
+		if (mysqlfile) {
+			fputs(querystr, mysqlfile);
+			fputs(";\n", mysqlfile);
+			fclose(mysqlfile);
+		}
+		return ret;
 	}
 
-          /* Got start record */
-          if(sqlrecord->AcctStatusTypeId == PW_STATUS_START) {
+	/* Got start record */
+	if(sqlrecord->AcctStatusTypeId == PW_STATUS_START) {
              
-             /* Set start time on record with only a stop record */
- 	     snprintf(querystr, 2048, "UPDATE %s SET AcctStartTime = '%s', AcctStartDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
-	     sql->config->sql_acct_table,
-             sqlrecord->AcctTimeStamp,
-	     sqlrecord->AcctDelayTime,
-             sqlrecord->AcctSessionId,
-             sqlrecord->UserName,
-             sqlrecord->NASIPAddress,
-             sqlrecord->NASPortId
-             );
-       	     if (sql_query(socket, querystr) < 0)
-	        log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", sql_error(socket));
-	     sql_finish_query(socket);
+		/* Set start time on record with only a stop record */
+		snprintf(querystr, 2048, "UPDATE %s SET AcctStartTime = '%s', AcctStartDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
+		sql->config->sql_acct_table,
+		sqlrecord->AcctTimeStamp,
+		sqlrecord->AcctDelayTime,
+		sqlrecord->AcctSessionId,
+		sqlrecord->UserName,
+		sqlrecord->NASIPAddress,
+		sqlrecord->NASPortId
+		);
 
-             num = sql_affected_rows(socket);
-             if (num == 0) {
-
-                /* Insert new record with blank stop time until stop record is got */
-                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', '%s', 0, 0, '%s', '%s', 0, 0, '%s', '%s', '', '%s', '%s', '%s', %ld, 0)",
-                sql->config->sql_acct_table,
-                sqlrecord->AcctSessionId,
-                sqlrecord->UserName,
-                sqlrecord->Realm,
-                sqlrecord->NASIPAddress,
-                sqlrecord->NASPortId,
-                sqlrecord->NASPortType,
-                sqlrecord->AcctTimeStamp,
-                sqlrecord->AcctAuthentic,
-                sqlrecord->ConnectInfo,
-                sqlrecord->CalledStationId,
-                sqlrecord->CallingStationId,
-                sqlrecord->ServiceType,
-                sqlrecord->FramedProtocol,
-                sqlrecord->FramedIPAddress,
-                sqlrecord->AcctDelayTime
-                );                  
-
-       	        if (sql_query(socket, querystr) < 0)
-	   	  log(L_ERR, "Acct: Couldn't insert SQL accounting START record - %s", sql_error(socket));
+		if (!sql_query(socket, querystr)) {
+			log(L_ERR, "Acct: Couldn't update SQL accounting START record - %s", sql_error(socket));
+			ret = 0;
+		}
 		sql_finish_query(socket);
-             }
+
+		num = sql_affected_rows(socket);
+		if (num == 0 && ret > 0) {
+
+			/* Insert new record with blank stop time until we receive the stop record */
+			snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', '%s', 0, 0, '%s', '%s', 0, 0, '%s', '%s', '', '%s', '%s', '%s', %ld, 0)",
+			sql->config->sql_acct_table,
+			sqlrecord->AcctSessionId,
+			sqlrecord->UserName,
+			sqlrecord->Realm,
+			sqlrecord->NASIPAddress,
+			sqlrecord->NASPortId,
+			sqlrecord->NASPortType,
+			sqlrecord->AcctTimeStamp,
+			sqlrecord->AcctAuthentic,
+			sqlrecord->ConnectInfo,
+			sqlrecord->CalledStationId,
+			sqlrecord->CallingStationId,
+			sqlrecord->ServiceType,
+			sqlrecord->FramedProtocol,
+			sqlrecord->FramedIPAddress,
+			sqlrecord->AcctDelayTime
+			);                  
+	
+			if (!sql_query(socket, querystr)) {
+				log(L_ERR, "Acct: Couldn't insert SQL accounting START record - %s", sql_error(socket));
+				ret = 0;
+			}
+			sql_finish_query(socket);
+		}
 
            /* Got stop record */
-           } else {
+	} else if (sqlrecord->AcctStatusTypeId == PW_STATUS_STOP) {
 
 		char	authstr[(AUTH_STRING_LEN * 2) + 50];
+
 		if (sql->config->sensitiveusername) 
 			sprintf(authstr, "UserName = '%s' AND STRCMP(UserName, '%s') = 0", sqlrecord->UserName, sqlrecord->UserName);
 		else
 			sprintf(authstr, "UserName = '%s'", sqlrecord->UserName);
-             sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND %s", sql->config->sql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, authstr);
-              sql_select_query(socket, querystr);
-              num = sql_num_rows(socket);
-	      sql_finish_select_query(socket);
 
-             if (num > 0) {
+		sprintf(querystr, "SELECT RadAcctId FROM %s WHERE AcctSessionId='%s' AND NASIPAddress='%s' AND %s", sql->config->sql_acct_table, sqlrecord->AcctSessionId, sqlrecord->NASIPAddress, authstr);
+		sql_select_query(socket, querystr);
+		num = sql_num_rows(socket);
+		sql_finish_select_query(socket);
 
-                /* Set stop time on matching record with start time */
- 	        snprintf(querystr, 2048, "UPDATE %s SET AcctStopTime = '%s', AcctSessionTime = '%lu', AcctInputOctets = '%lu', AcctOutputOctets = '%lu', AcctTerminateCause = '%s', AcctStopDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
-	        sql->config->sql_acct_table,
-                sqlrecord->AcctTimeStamp,
-                sqlrecord->AcctSessionTime,
-                sqlrecord->AcctInputOctets,
-                sqlrecord->AcctOutputOctets,
-                sqlrecord->AcctTerminateCause,
-		sqlrecord->AcctDelayTime,
-                sqlrecord->AcctSessionId,
-                sqlrecord->UserName,
-                sqlrecord->NASIPAddress,
-                sqlrecord->NASPortId
-                );
+		if (num > 0) {
 
+			/* Set stop time on matching record with start time */
+			snprintf(querystr, 2048, "UPDATE %s SET AcctStopTime = '%s', AcctSessionTime = '%lu', AcctInputOctets = '%lu', AcctOutputOctets = '%lu', AcctTerminateCause = '%s', AcctStopDelay = %ld WHERE AcctSessionId = '%s' AND UserName = '%s' AND NASIPAddress = '%s' AND NASPortId = '%ld'", 
+			sql->config->sql_acct_table,
+			sqlrecord->AcctTimeStamp,
+			sqlrecord->AcctSessionTime,
+			sqlrecord->AcctInputOctets,
+			sqlrecord->AcctOutputOctets,
+			sqlrecord->AcctTerminateCause,
+			sqlrecord->AcctDelayTime,
+			sqlrecord->AcctSessionId,
+			sqlrecord->UserName,
+			sqlrecord->NASIPAddress,
+			sqlrecord->NASPortId
+			);
 
-       	        if (sql_query(socket, querystr) < 0)
-	           log(L_ERR, "Acct: Couldn't update SQL accounting STOP record - %s", sql_error(socket));
-		sql_finish_query(socket);
+			if (!sql_query(socket, querystr)) {
+				log(L_ERR, "Acct: Couldn't update SQL accounting STOP record - %s", sql_error(socket));
+				ret = 0;
+			}
+			sql_finish_query(socket);
 
-             } else if (num == 0) {
+		} else if (num == 0) {
 
 #ifdef CISCO_ACCOUNTING_HACK
-		/* If stop but zero session length AND no previous session found, drop it as in invalid packet */
-		/* This is to fix CISCO's aaa from filling our table with bogus crap */
-		if (sqlrecord->AcctSessionTime <= 0) {
-			log(L_ERR, "Acct: Invalid STOP record. [%s] STOP record but zero session length? (nas %s)", sqlrecord->UserName, sqlrecord->NASIPAddress);
-			return 0;
-		}
-#endif
+			/* If stop but zero session length AND no previous session found, drop it as in invalid packet
+			 * This is to fix CISCO's aaa from filling our table with bogus crap */
+			if (sqlrecord->AcctSessionTime <= 0) {
+				log(L_ERR, "Acct: Invalid STOP record. [%s] STOP record but zero session length? (nas %s)", sqlrecord->UserName, sqlrecord->NASIPAddress);
+				if(mysqlfile) {
+					fclose(mysqlfile);
+				}
+				/* Return 1 so that we send a ACK to the NAS to keep it from resending over and over */
+				return 1;
+			}
+#endif /* CISCO_ACCOUNTING_HACK */
             
-                /* Insert record with no start time until matching start record comes */
-                snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', 0, '%s', '%lu', '%s', '%s', '%lu', '%lu', '%s', '%s', '%s', '%s', '%s', '%s', 0, %ld)",
-                sql->config->sql_acct_table,
-                sqlrecord->AcctSessionId,
-                sqlrecord->UserName,
-                sqlrecord->Realm,
-                sqlrecord->NASIPAddress,
-                sqlrecord->NASPortId,
-                sqlrecord->NASPortType,
-	        sqlrecord->AcctTimeStamp,
-		sqlrecord->AcctSessionTime,
-                sqlrecord->AcctAuthentic,
-                sqlrecord->ConnectInfo,
-		sqlrecord->AcctInputOctets,
-		sqlrecord->AcctOutputOctets,
-                sqlrecord->CalledStationId,
-                sqlrecord->CallingStationId,
-		sqlrecord->AcctTerminateCause,
-                sqlrecord->ServiceType,
-                sqlrecord->FramedProtocol,
-                sqlrecord->FramedIPAddress,
-                sqlrecord->AcctDelayTime
-                );                  
+			/* Insert record with no start time until matching start record comes */
+			snprintf(querystr, 2048, "INSERT INTO %s VALUES (0, '%s', '%s', '%s', '%s', %ld, '%s', 0, '%s', '%lu', '%s', '%s', '%lu', '%lu', '%s', '%s', '%s', '%s', '%s', '%s', 0, %ld)",
+			sql->config->sql_acct_table,
+			sqlrecord->AcctSessionId,
+			sqlrecord->UserName,
+			sqlrecord->Realm,
+			sqlrecord->NASIPAddress,
+			sqlrecord->NASPortId,
+			sqlrecord->NASPortType,
+			sqlrecord->AcctTimeStamp,
+			sqlrecord->AcctSessionTime,
+			sqlrecord->AcctAuthentic,
+			sqlrecord->ConnectInfo,
+			sqlrecord->AcctInputOctets,
+			sqlrecord->AcctOutputOctets,
+			sqlrecord->CalledStationId,
+			sqlrecord->CallingStationId,
+			sqlrecord->AcctTerminateCause,
+			sqlrecord->ServiceType,
+			sqlrecord->FramedProtocol,
+			sqlrecord->FramedIPAddress,
+			sqlrecord->AcctDelayTime
+			);                  
 
-       	        if (sql_query(socket, querystr) < 0)
-		   log(L_ERR, "Acct: Couldn't insert SQL accounting STOP record - %s", sql_error(socket));
-		sql_finish_query(socket);
-             }
+			if (!sql_query(socket, querystr)) {
+				log(L_ERR, "Acct: Couldn't insert SQL accounting STOP record - %s", sql_error(socket));
+				ret = 0;
+			}
+			sql_finish_query(socket);
+		}
 
-          }
-          if (mysqlfile) {
-                fputs(querystr, mysqlfile);
-                fputs(";\n", mysqlfile);
-                fflush(mysqlfile);
-                fclose(mysqlfile);
-          }
+	} else {
+		log(L_ERR, "Acct: Unknown Accounting status type %d", sqlrecord->AcctStatusTypeId);
+		return 0;
+	}
 
-     return 0;
+	if (mysqlfile) {
+		fputs(querystr, mysqlfile);
+		fputs(";\n", mysqlfile);
+		fflush(mysqlfile);
+		fclose(mysqlfile);
+	}
 
+	return ret;
 }
 
 
@@ -648,6 +668,9 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 	VALUE_PAIR	*pair, *pair2, *check;
 	struct tm	*tm;
 	time_t		timeval;
+#if defined( BINARY_FILTERS )
+	char		valstr[256];
+#endif	/* BINARY_FILTERS */
 
 
 	if((attr = dict_attrfind(row[2])) == (DICT_ATTR *)NULL) {
@@ -672,18 +695,22 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 		}
 		
 		/* HINTS items ALWAYS replace user or group items */
-		if (hintsfound && (ufound || gfound)) 
-			return 0;
+		if (hintsfound && (ufound || gfound)) {
+		   DEBUG2("attribute %s for user %s already in hints", row[2], row[1]);
+			return 1;
+                }
 
-		if (ufound && gfound)
-			return 0;
+		if (ufound && gfound) {
+		   DEBUG2("attribute %s for user %s already ufound & gfound ", row[2], row[1]);
+			return 1;
+		}
 
 		check = check->next;
 	}
 
 	if((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) == (VALUE_PAIR *)NULL) {
 		log(L_CONS|L_ERR, "mysql_userparse: no memory");
-		exit(1);
+		return -2;
 	}
 	strcpy(pair->name, attr->name);
 	pair->attribute = attr->value;
@@ -692,6 +719,19 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 	pair->source = mode;
 	switch(pair->type) {
 
+#if defined( BINARY_FILTERS )
+		case PW_TYPE_FILTER_BINARY:
+			/*
+			* special case to convert filter to binary
+			*/
+			strcpy(valstr, row[3]);
+			if ( filterBinary( pair, valstr ) == -1 ) {
+				free(pair);
+				return(-1);
+			}
+			break;
+#endif	/* BINARY_FILTERS */
+
 		case PW_TYPE_STRING:
 			strcpy(pair->strvalue, row[3]);
 			pair->length = strlen(pair->strvalue);
@@ -754,7 +794,7 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
                         */
                         if((pair2 = malloc(sizeof(VALUE_PAIR))) == NULL) {
                                log(L_CONS|L_ERR, "no memory");
-                               exit(1);
+                               return -2;
                         }
                         strcpy(pair2->name, "Add-Port-To-IP-Address");
                         pair2->attribute = PW_ADD_PORT_TO_IP_ADDRESS;
@@ -783,7 +823,7 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 	}
 	pairadd(first_pair, pair);
 
-	return 0;
+	return 1;
 }
 
 
@@ -797,7 +837,7 @@ int sql_userparse(VALUE_PAIR **first_pair, SQL_ROW row, int mode) {
 int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int mode) {
 
 	char		*querystr;
-	char		*authstr;
+	char		*authstr = NULL;
 	char		*username;
 	SQL_ROW		row;
 	int		rows;
@@ -817,6 +857,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 
 	if (mode == PW_VP_USERDATA) {
 		if ((authstr = malloc((strlen(username) * 2) + 50)) == NULL) {
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -826,6 +867,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 			sprintf(authstr, "UserName = '%s'", username);
 		if ((querystr = malloc(strlen(authstr) + strlen(table) + 70)) == NULL) {
 			free(authstr);
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -833,6 +875,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 
 	} else if (mode == PW_VP_GROUPDATA) {
 		if ((authstr = malloc((strlen(username) * 2) + strlen(sql->config->sql_usergroup_table) + 55)) == NULL) {
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -842,6 +885,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 			sprintf(authstr, "%s.UserName = '%s'",sql->config->sql_usergroup_table, username);
 		if ((querystr = malloc(strlen(authstr) + (strlen(table) * 7) + (strlen(sql->config->sql_usergroup_table) * 2) + 125)) == NULL) {
 			free(authstr);
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -849,6 +893,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 
 	} else if (mode == PW_VP_REALMDATA) {
 		if ((querystr = malloc(strlen(username) + (strlen(table) * 7) + (strlen(sql->config->sql_realmgroup_table) * 3) + 125)) == NULL) {
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -856,6 +901,7 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 	} else if (mode == PW_VP_HINTSDATA) {
 		if ((querystr = malloc(strlen(username) + strlen(table) + 125)) == NULL) {
 			free(authstr);
+			free(username);
 			log(L_CONS|L_ERR, "out of memory");
 			return -1;
 		}
@@ -867,17 +913,18 @@ int sql_getvpdata(SQLSOCK *socket, char *table, VALUE_PAIR **vp, char *user, int
 	rows = sql_num_rows(socket);
 	while ((row = sql_fetch_row(socket))) {
 
-		if (sql_userparse(vp, row, mode) != 0) {
-	 		log(L_ERR|L_CONS, "sql_userparse: Error adding row to pair list");
+		if (!sql_userparse(vp, row, mode)) {
+	 		log(L_ERR|L_CONS, "sql_userparse: [%s] Error adding attribute [%s] with value [%s] to pair list", row[1], row[2], row[3]);
 			sql_finish_select_query(socket);
 			free(querystr);
-			free(authstr);
+			if (authstr) free(authstr);
+			free(username);
 			return -1;
 		}
 	}
 	sql_finish_select_query(socket);
 	free(querystr);
-	free(authstr);
+	if (authstr) free(authstr);
 	free(username);
 
 	return rows;
@@ -1041,8 +1088,11 @@ int sql_check_multi(SQLSOCK *socket, char *name, VALUE_PAIR *request, int maxsim
 				if ((chldpid = fork()) == 0) {
 					sock = sql_create_socket();
 					sprintf(querystr, "DELETE FROM %s WHERE RadAcctId = '%s'", sql->config->sql_acct_table, row[0]);
-					sql_query(sock, querystr);
-					sql_finish_query(sock);
+					if (!sql_query(sock, querystr)) {
+						log(L_ERR, "Could not delete stale session '%s'", row[0]);
+					} else {
+						sql_finish_query(sock);
+					}
 					sql_close_socket(sock);
 					exit(1);
 				} else {
@@ -1181,7 +1231,7 @@ int sql_read_naslist(SQLSOCK *socket) {
        	}
 	sql_finish_select_query(socket);
 
-        return 0;
+        return 1;
 } 
 
 
@@ -1284,6 +1334,11 @@ int sql_dict_init(SQLSOCK *socket) {
 			else if(strcmp(row[4], "date") == 0) {
 				type = PW_TYPE_DATE;
 			}
+#if defined( BINARY_FILTERS )
+			else if(strcmp(row[4], "abinary") == 0) {
+				type = PW_TYPE_FILTER_BINARY;
+			}
+#endif	/* BINARY_FILTERS */
 			else {
 				log(L_ERR|L_CONS,
 			"dict_init: Invalid type on line %d of dictionary",
@@ -1333,33 +1388,26 @@ int sql_dict_init(SQLSOCK *socket) {
 			 * Validate all entries
 			 */
 			if(strlen(row[2]) > 31) {
-				log(L_ERR|L_CONS,
-		"dict_init: Invalid attribute length on line %d of dictionary",
-					line_no);
+				log(L_ERR|L_CONS, "dict_init: Invalid attribute length on line %d of dictionary", line_no);
 				sql_finish_select_query(socket);
 				return(-1);
 			}
 
 			if(strlen(row[3]) > 31) {
-				log(L_ERR|L_CONS,
-		"dict_init: Invalid name length on line %d of dictionary",
-					line_no);
+				log(L_ERR|L_CONS, "dict_init: Invalid name length on line %d of dictionary", line_no);
 				sql_finish_select_query(socket);
 				return(-1);
 			}
 
 			if(!isdigit(*row[4])) {
-				log(L_ERR|L_CONS,
-			"dict_init: Invalid value on line %d of dictionary",
-					line_no);
+				log(L_ERR|L_CONS, "dict_init: Invalid value on line %d of dictionary", line_no);
 				sql_finish_select_query(socket);
 				return(-1);
 			}
 			value = strtol(row[4], NULL, 0);
 
 			/* Create a new VALUE entry for the list */
-			if((dval = (DICT_VALUE *)malloc(sizeof(DICT_VALUE))) ==
-					(DICT_VALUE *)NULL) {
+			if((dval = (DICT_VALUE *)malloc(sizeof(DICT_VALUE))) == (DICT_VALUE *)NULL) {
 				log(L_ERR|L_CONS, "dict_init: out of memory");
 				sql_finish_select_query(socket);
 				return(-1);
@@ -1379,17 +1427,13 @@ int sql_dict_init(SQLSOCK *socket) {
 			 * Validate all entries
 			 */
 			if(strlen(row[2]) > 31) {
-				log(L_ERR|L_CONS,
-		"dict_init: Invalid attribute length on line %d of dictionary",
-					line_no);
+				log(L_ERR|L_CONS, "dict_init: Invalid attribute length on line %d of dictionary", line_no);
 				sql_finish_select_query(socket);
 				return(-1);
 			}
 
 			if(!isdigit(*row[3])) {
-				log(L_ERR|L_CONS,
-			"dict_init: Invalid value on line %d of dictionary",
-					line_no);
+				log(L_ERR|L_CONS, "dict_init: Invalid value on line %d of dictionary", line_no);
 				sql_finish_select_query(socket);
 				return(-1);
 			}
@@ -1407,7 +1451,7 @@ int sql_dict_init(SQLSOCK *socket) {
 		}
 	}
 	sql_finish_select_query(socket);
-	return(0);
+	return(1);
 }
 
  /*
@@ -1448,7 +1492,7 @@ int sql_read_realms(SQLSOCK *socket) {
 		strNcpy(c->realm, row[0], sizeof(c->realm));
 	if (c->server != NULL)
 		strNcpy(c->server, row[1], sizeof(c->server));
-	if (strcmp(row[1], "LOCAL") != 0)
+	if (strcmp(row[2], "") == 0)
 		c->ipaddr = get_ipaddr(row[1]);
 	else
 		c->ipaddr = ipstr2long(row[2]);
