@@ -31,6 +31,8 @@ static		char *send_buffer = (char *)i_send_buffer;
 int		proxy_id = 1;
 AUTH_REQ	*proxy_requests;
 
+extern int trust_proxies;
+
 static int allowed [] = {
 	PW_SERVICE_TYPE,
 	PW_FRAMED_PROTOCOL,
@@ -81,11 +83,7 @@ static void proxy_cleanup(void)
 			else
 				proxy_requests = a->next;
 
-			if (a->data) free(a->data);
-			if (a->server_reply) pairfree(a->server_reply);
-			if (a->request) pairfree(a->request);
-			if (a->proxy_pairs) pairfree(a->proxy_pairs);
-			free(a);
+			authfree(a);
 
 			continue;
 		}
@@ -147,9 +145,7 @@ int proxy_addrequest(AUTH_REQ *authreq, int *proxy_id)
 			last->next = a->next;
 		else
 			proxy_requests = a->next;
-		if (a->data) free(a->data);
-		if (a->server_reply) pairfree(a->server_reply);
-		free(a);
+		authfree(a);
 	}
 	if (id < 0) {
 		id = (*proxy_id)++;
@@ -254,6 +250,13 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	 */
 	proxy_cleanup();
 
+#if defined(ALL_ACCOUNTING_LOCAL)
+	if (authreq->code == PW_ACCOUNTING_REQUEST) {
+		return 0;
+	}
+#endif /* ALL_ACCOUNTING_LOCAL */
+
+
 	/*
 	 *	Look up name.
 	 */
@@ -320,6 +323,13 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	else
 		rport = realm->acct_port;
 
+	if (rport == 0) {
+		/* Don't want to proxy this type of request */
+		strcpy(namepair->strvalue, saved_username);
+		return 0;
+	}
+
+
 	secret_key = client->secret;
 
 	authreq->server_ipaddr = realm->ipaddr;
@@ -640,17 +650,21 @@ int proxy_receive(AUTH_REQ *authreq, int activefd) {
 			prev->next = last->next;
 		else
 			authreq->proxy_pairs = last->next;
+		free(last);
 	}
 
 	/*
 	 *	Only allow some attributes to be propagated from
 	 *	the remote server back to the NAS, for security.
 	 */
-	allowed_pairs = NULL;
-	for(i = 0; allowed[i]; i++)
-		pairmove2(&allowed_pairs, &(authreq->request), allowed[i]);
-	pairfree(authreq->request);
-
+        if (trust_proxies != 0) {
+	   allowed_pairs = authreq->request;
+        } else {
+	   allowed_pairs = NULL;
+	   for(i = 0; allowed[i]; i++)
+	      pairmove2(&allowed_pairs, &(authreq->request), allowed[i]);
+	   pairfree(authreq->request);
+	}
 	/*
 	 *	Now rebuild the AUTHREQ struct, so that the
 	 *	normal functions can process it.
@@ -663,6 +677,8 @@ int proxy_receive(AUTH_REQ *authreq, int activefd) {
 	oldreq->data = authreq->data;
 	memcpy(authreq, oldreq, sizeof(AUTH_REQ));
 
+	free(oldreq);
+
 	return 0;
 }
 
