@@ -22,12 +22,48 @@ char misc_sccsid[] =
 
 #include	"radiusd.h"
 
+/*RELAX start*/
+u_char    rrnr[32];
+u_char    rrnrpool[33];
+u_char    clid_rrnr[32];
+/*RELAX stop*/
+
 /*
  *	Make sure our buffer is aligned.
  */
 static int	i_send_buffer[1024];
 static char	*send_buffer = (char *)i_send_buffer;
 
+#ifdef ASCEND_SECRET
+/*************************************************************************
+ *
+ *      Function: make_secret
+ *
+ *      Purpose: Build an encrypted secret value to return in a reply
+ *               packet.  The secret is hidden by xoring with a MD5 digest
+ *               created from the shared secret and the authentication
+ *               vector.  We put them into MD5 in the reverse order from
+ *               that used when encrypting passwords to RADIUS.
+ *
+ *************************************************************************/
+
+void make_secret(u_char *digest, u_char *vector, u_char *secret, char *value)
+{
+        u_char  buffer[ AUTH_STRING_LEN ];
+        int             secretLen = strlen( (const char *)secret );
+        int             ix;
+
+        memcpy( buffer, vector, AUTH_VECTOR_LEN );
+        memcpy( buffer + AUTH_VECTOR_LEN, secret, secretLen );
+        md5_calc( digest, buffer, AUTH_VECTOR_LEN + secretLen );
+        memset( buffer, 0, AUTH_STRING_LEN );
+        for ( ix = 0; ix < AUTH_VECTOR_LEN; ix += 1 ) {
+                digest[ ix ] ^= value[ ix ];
+        }
+}
+#endif /* ASCEND_SECRET */
+
+
 /*
  *	Reply to the request.  Also attach
  *	reply attribute value pairs and any user message provided.
@@ -142,11 +178,40 @@ int rad_send_reply(int code, AUTH_REQ *authreq, VALUE_PAIR *oreply,
 		switch(reply->type) {
 
 		case PW_TYPE_STRING:
+			/*
+			* This is to hash the Ascend-Send-Secret before
+			* we send it to the Max.
+			*/
+#ifdef ASCEND_SECRET
+			if (( strcmp(reply->name, "Ascend-Send-Secret") == 0 )
+			|| ( strcmp(reply->name, "Ascend-Recv-Secret") == 0 )) {
+				make_secret( digest, authreq->vector,
+				authreq->secret, reply->strvalue );
+				*ptr++ = AUTH_VECTOR_LEN + 2;
+				memcpy( ptr, digest, AUTH_VECTOR_LEN );
+				ptr += AUTH_VECTOR_LEN;
+				total_length += AUTH_VECTOR_LEN + 2;
+				break;
+			}
+#endif	/* ASCEND_SECRET */
+
 			/*
 			 *	FIXME: this is just to make sure but
 			 *	should NOT be needed. In fact I have no
 			 *	idea if it is needed :)
 			 */
+
+			/*RELAX start*/
+			if (strcmp(reply->name,"Ascend-Dial-Number")== 0
+				&&rrnr[0]!='\0')
+			{
+				sprintf(rrnrpool,"9%s",rrnr);
+				strcpy(reply->name,"pusegn");
+				strcpy(reply->strvalue,rrnrpool);
+				reply->length = strlen(reply->strvalue);
+			}
+			/*RELAX stop */
+
 			if (reply->length == 0 && reply->strvalue[0] != 0)
 				reply->length = strlen(reply->strvalue);
 
@@ -177,6 +242,18 @@ int rad_send_reply(int code, AUTH_REQ *authreq, VALUE_PAIR *oreply,
 			total_length += sizeof(UINT4) + 2;
 			break;
 
+#if defined( BINARY_FILTERS )
+		case PW_TYPE_FILTER_BINARY:
+			/* The binary representation of the filter is in
+			reply->strvalue.  It's length is in reply->lvalue */
+
+			*ptr++ = reply->lvalue + 2;
+			memcpy( ptr, reply->strvalue, reply->lvalue );
+			ptr += reply->lvalue;
+			total_length += reply->lvalue + 2;
+			break;
+#endif	/* BINARY_FILTERS */
+
 		default:
 			break;
 		}
@@ -455,39 +532,45 @@ AUTH_REQ	*radrecv(UINT4 host, u_short udp_port,
 			}
 		}
 
-		if ((attr = dict_attrget(attribute)) == (DICT_ATTR *)NULL) {
-			DEBUG("Received unknown attribute %d", attribute);
+		if ((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) == (VALUE_PAIR *)NULL) {
+			log(L_ERR|L_CONS, "no memory");
+			exit(1);
 		}
-		else if ( attrlen >= AUTH_STRING_LEN ) {
+		memset(pair, 0, sizeof(VALUE_PAIR));
+
+		if ((attr = dict_attrget(attribute)) == (DICT_ATTR *)NULL) {
+			sprintf(pair->name, "Unknown-Attr-%d", attribute);
+			pair->type = PW_TYPE_STRING;
+		} else {
+			strcpy(pair->name, attr->name);
+			pair->type = attr->type;
+		}
+
+		if ( attrlen >= AUTH_STRING_LEN-1 ) {
 			DEBUG("attribute %d too long, %d >= %d", attribute,
 				attrlen, AUTH_STRING_LEN);
+			free(pair);
 		}
 		else if ( attrlen > length ) {
 			DEBUG("attribute %d longer as buffer left, %d > %d",
 				attribute, attrlen, length);
+			free(pair);
 		}
 		else {
-			if ((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) ==
-						(VALUE_PAIR *)NULL) {
-				log(L_ERR|L_CONS, "no memory");
-				exit(1);
-			}
-			/* Same size */
-			strcpy(pair->name, attr->name);
-			pair->attribute = attr->value;
-			pair->type = attr->type;
+			pair->attribute = attribute;
 			pair->length = attrlen;
 			pair->next = (VALUE_PAIR *)NULL;
 			pair->operator = PW_OPERATOR_EQUAL;
 			pair->strvalue[0] = '\0';
 
-			switch (attr->type) {
+			switch (pair->type) {
 
 			case PW_TYPE_STRING:
 				/* attrlen always < AUTH_STRING_LEN */
 				memset(pair->strvalue, 0, AUTH_STRING_LEN);
 				memcpy(pair->strvalue, ptr, attrlen);
 				debug_pair(stdout, pair);
+
 				if(first_pair == (VALUE_PAIR *)NULL) {
 					first_pair = pair;
 				}
@@ -510,10 +593,24 @@ AUTH_REQ	*radrecv(UINT4 host, u_short udp_port,
 				}
 				prev = pair;
 				break;
+
+#if defined( BINARY_FILTERS )
+			case PW_TYPE_FILTER_BINARY:
+				memcpy(pair->strvalue, ptr, attrlen);
+				pair->strvalue[attrlen] = '\0';
+				if(first_pair == (VALUE_PAIR *)NULL) {
+					first_pair = pair;
+				}
+				else {
+					prev->next = pair;
+				}
+				prev = pair;
+ 				break;
+#endif	/* BINARY_FILTERS */
 			
 			default:
 				DEBUG("    %s (Unknown Type %d)",
-					attr->name,attr->type);
+					pair->name,pair->type);
 				free(pair);
 				break;
 			}
@@ -529,6 +626,21 @@ AUTH_REQ	*radrecv(UINT4 host, u_short udp_port,
         /* hide the proxy pairs from the rest of the server */
         pairmove2(&authreq->proxy_pairs, &authreq->request, PW_PROXY_STATE);
 
+	/*
+	 *      Create a Client-IP-Address attribute, and add it to the
+	 *      user's request.
+	 */
+	if ((pair = malloc(sizeof(VALUE_PAIR))) == NULL) {
+		log(L_CONS|L_ERR, "no memory");
+		exit(1);
+	}
+	memset(pair, 0, sizeof(VALUE_PAIR));
+	strcpy(pair->name, "Client-IP-Address");
+	pair->attribute = PW_CLIENT_IP_ADDRESS;
+	pair->type = PW_TYPE_IPADDR;
+	pair->lvalue = host;
+	pairadd(&authreq->request, pair);
+
 	return(authreq);
 }
 
@@ -565,7 +677,7 @@ static void send_challenge(AUTH_REQ *authreq, char *msg,
 	 */
 	if (msg != (char *)NULL) {
 		len = strlen(msg);
-		if(len > 0 && len < AUTH_STRING_LEN) {
+		if(len > 0 && len < AUTH_STRING_LEN-1) {
 			ptr = auth->data;
 			*ptr++ = PW_REPLY_MESSAGE;
 			*ptr++ = len + 2;
@@ -618,5 +730,3 @@ static void send_challenge(AUTH_REQ *authreq, char *msg,
 			&saremote, sizeof(struct sockaddr_in));
 }
 #endif
-
-
