@@ -51,6 +51,7 @@ char radiusd_sccsid[] =
 #  include	<malloc.h>
 #  include	<getopt.h>
 #endif
+#include	<pwd.h>
 
 #include	"radiusd.h"
 
@@ -74,6 +75,8 @@ int			auth_port;
 int			acct_port;
 int			acct_pid;
 int			radius_pid;
+int                     trust_proxies = 0;
+char*                   accounting_method = "s";
 
 /*
  *	Make sure recv_buffer is aligned properly.
@@ -120,7 +123,7 @@ static void reread_config(int reload) {
 	}
 
 
-	if (sql_init(reload) != 0) {
+	if (!sql_init(reload)) {
 		if (!reload) 
 			log(L_ERR,"SQL Error: SQL connections could not be initialized");
 		else
@@ -131,7 +134,7 @@ static void reread_config(int reload) {
 	socket = sql_get_socket();
 
 	/* Initialize the dictionary */
-	if (sql_dict_init(socket) != 0)
+	if (!sql_dict_init(socket))
 		res = -1;
 
 	/* Initialize Configuration Values */
@@ -175,6 +178,9 @@ int main(int argc, char **argv)
 	int			dontfork = 0;
 	int			radius_port = 0;
 	SQLSOCK			*acctsock;
+        int                     run_as_uid = 0;	
+        int                     run_as_gid = 0;	
+	char	                *chroot_dir = NULL;	
 #ifdef RADIUS_PID
 	FILE			*fp;
 #endif
@@ -220,7 +226,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:snvxyz")) != EOF) {
+	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:m:r:sntu:vxyz")) != EOF) {
 
 		switch(argval) {
 
@@ -260,6 +266,10 @@ int main(int argc, char **argv)
 		case 'l':
 			radlog_dir = optarg;
 			break;
+			
+		case 'm':
+  		        accounting_method = optarg;
+			break;
 		
 		case 'S':
 			log_stripped_names++;
@@ -269,10 +279,31 @@ int main(int argc, char **argv)
 			radius_port = atoi(optarg);
 			break;
 
+		case 'r':
+			chroot_dir = optarg;
+			break;
+		
 		case 's':	/* Single process mode */
 			spawn_flag = 0;
 			break;
 
+		case 't':
+		        trust_proxies = 1;
+		        break;
+
+                case 'u':
+                {
+                       struct passwd *pw;
+                       if ((pw = getpwnam(optarg)) == 0) {
+                                fprintf(stderr, "radiusd: %s: user unknown\n",
+                                        optarg);
+                                exit(1);
+                        }
+
+                        run_as_uid = pw->pw_uid;
+			run_as_gid = pw->pw_gid;
+                        break;
+                }
 		case 'v':
 			version();
 			break;
@@ -295,6 +326,31 @@ int main(int argc, char **argv)
 		}
 	}
 
+	/*
+	 *	Change root dir
+	 */
+	if (chroot_dir != NULL) {
+	   if (chroot(chroot_dir) != 0) {
+	      fprintf(stderr, "radiusd: could not chroot to %s: %s\n", chroot_dir, strerror(errno));
+	      exit(1);
+	   }
+	}	
+
+	/* 
+         *	Change user ID
+	 */
+	if (run_as_uid != 0) {
+           if (setgid(run_as_gid) != 0) {
+		fprintf(stderr, "could not set gid to %d", run_as_gid);
+		exit(1);
+	   }
+           if (setuid(run_as_uid) != 0) {
+                fprintf(stderr, "could not set uid to %d", run_as_uid);
+                exit(1);
+           }
+	}
+
+          
 	if ((sql = malloc(sizeof(SQL))) == NULL) {
 		log(L_ERR|L_CONS, "no memory");
 		exit(1);
@@ -429,8 +485,15 @@ int main(int argc, char **argv)
 			close(sockfd);
 			sockfd = -1;
 		}
-	} else
+	} else {
+	        acct_pid = getpid();
+		acctsock = sql_create_socket();
+		acctsock->id = sql->config->max_sql_socks;
+		acctsock->in_use = getpid();
+		DEBUG2("SQL: Connected socket %d", sql->config->max_sql_socks);
 		log(L_INFO, "Ready to process requests.");
+		acct_pid = 0;
+	}
 
 
 	/*
@@ -635,13 +698,11 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun, SQLSOCK *
 			 */
 			if (prevreq == (AUTH_REQ *)NULL) {
 				first_request = curreq->next;
-				pairfree(curreq->request);
-				free(curreq);
+				authfree(curreq);
 				curreq = first_request;
 			} else {
 				prevreq->next = curreq->next;
-				pairfree(curreq->request);
-				free(curreq);
+				authfree(curreq);
 				curreq = prevreq->next;
 			}
 		} else if (curreq->ipaddr == authreq->ipaddr &&
@@ -657,8 +718,7 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun, SQLSOCK *
 				log(L_ERR, "Dropping duplicate authentication packet from client %s - ID: %d",
 				client_name(authreq->ipaddr), authreq->id);
 				sql_release_socket(socket);
-				pairfree(authreq->request);
-				free(authreq);
+				authfree(authreq);
 				request_list_busy = 0;
 				sig_cleanup(SIGCHLD);
 
@@ -709,8 +769,7 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun, SQLSOCK *
 	if (request_count > MAX_REQUESTS) {
 		log(L_ERR, "Dropping request (too many): from client %s - ID: %d",
 				client_name(authreq->ipaddr), authreq->id);
-		pairfree(authreq->request);
-		free(authreq);
+		authfree(authreq);
 
 		request_list_busy = 0;
 		sig_cleanup(SIGCHLD);
