@@ -238,6 +238,9 @@ int		rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul);
 void		fprint_attr_list(FILE *, VALUE_PAIR *);
 void		fprint_attr_val(FILE *, VALUE_PAIR *);
 
+void		fprint_attr_list_csv(FILE *, VALUE_PAIR *, int);
+void		fprint_attr_val_csv(FILE *, VALUE_PAIR *);
+
 /* dict.c */
 int		dict_init(char *);
 DICT_ATTR	*dict_attrget(int);
@@ -272,7 +275,7 @@ VALUE_PAIR	*pairfind(VALUE_PAIR *, int);
 void		pairdelete(VALUE_PAIR **, int);
 void		pairadd(VALUE_PAIR **, VALUE_PAIR *);
 void		authfree(AUTH_REQ *authreq);
-#if defined (sun) && defined(__svr4__)
+#if defined (sun) && defined(__svr4__) || defined(__hpux__) || defined(aix)
 void		(*sun_signal(int signo, void (*func)(int)))(int);
 #define signal sun_signal
 #endif
@@ -313,6 +316,13 @@ void		user_gettime(char *valstr, struct tm *tm);
 /* timestr.c */
 int		timestr_match(char *, time_t);
 
+#if defined( BINARY_FILTERS )
+void            pairmove2(VALUE_PAIR **to, VALUE_PAIR **from, int attr);
+
+/* filters.c */
+int            filterBinary(VALUE_PAIR *pair, char *valstr);
+#endif	/* BINARY_FILTERS */
+ 
 /* version.c */
 void		version();
 
