@@ -26,7 +26,7 @@
  */
 
 char version_sccsid[] =
-"@(#)version.c	1.3 Copyright 1992 Livingston Enterprises Inc";
+"@(#)version.c	1.4 Copyright 1992 Livingston Enterprises Inc";
 
 #include        <sys/types.h>
 #include	<stdio.h>
@@ -37,7 +37,7 @@ char version_sccsid[] =
  *  would like a range of versions allocated for your use.
  */
 
-#define		VERSION		"0.16"
+#define		VERSION		"0.17b"
 
 /*
  *	Display the revision number for this program
@@ -64,6 +64,12 @@ void version(void)
 #if defined(CISCO_ACCOUNTING_HACK)
 	fprintf(stderr, " CISCO_ACCOUNTING_HACK");
 #endif
+#if defined(BINARY_FILTERS)
+	fprintf(stderr, " BINARY_FILTERS");
+#endif
+#if defined(ASCEND_SECRET)
+	fprintf(stderr, " ASCEND_SECRET");
+#endif
 #if defined(NOCASE)
 	fprintf(stderr," NOCASE");
 #endif
